#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""python pac currency converter."""

# imports ####################################################################

import sys

from Tkinter import *
root = Tk()

sys.path.append("..")
import PAC


# multiple view ##############################################################

class MoneyMVC(PAC.Controller):
	"""money controller with multiple view."""
	
	def __init__(self, parent=None):
		PAC.Controller.__init__(self, 
		                        parent=parent,
		                        abstraction=MoneyA(self),
		                        presentation=MoneyMVP(self))
		self.children.append(MoneyC(self))
		self.update()
		
	def update(self):
		for child in self.children:
			child.update()
			
	def get_amount(self): 
		return self.abstraction.amount
	def set_amount(self, amount): 
		self.abstraction.amount = amount
	amount = property(get_amount, set_amount)
	
	def get_frame(self):
		return self.presentation.get_frame()


class MoneyMVP(PAC.Presentation):
	"""multiple view money presentation (single window)."""

	def __init__(self, controller=None, root=None):
		PAC.Presentation.__init__(self, controller=controller)
		if root == None:
			root = Toplevel()
		self._root = root
		self._root.title("0-converter")
		self._root.protocol("WM_DELETE_WINDOW", sys.exit)
		
	def get_frame(self):
		frame = Frame(self._root)
		frame.pack(expand=True, fill=X)
		return frame
	
	
class MoneyA(PAC.Abstraction):
	"""Money abstraction."""
	
	def __init__(self, controller=None, amount=0):
		PAC.Abstraction.__init__(self, controller)
		self._amount = amount
		
	def get_amount(self):
		return self._amount
	def set_amount(self, amount):
		self._amount = amount
		self.controller.update()
	amount = property(get_amount, set_amount)

		
# money ######################################################################

class MoneyC(PAC.Controller):
	"""money controller."""
	
	def __init__(self, parent=None, unit="€", rate=1.):
		PAC.Controller.__init__(self,
		                        parent=parent,
		                        presentation=ValueP(self, unit=unit, frame=parent.get_frame()),
		                        abstraction=RateA(self, rate))
		self._unit = unit
		self.update()
		
	def update(self):
		self.presentation.value = self.amount
		
	def edit_rate(self):
		RateEditorC(self, self._unit)
	
	def get_rate(self):
		return self.abstraction.rate
	def set_rate(self, rate):
		self.abstraction.rate = rate
	rate = property(get_rate, set_rate)
	
	def get_amount(self): 
		return self.parent.amount * self.rate
	def set_amount(self, amount): 
		self.parent.amount = amount / self.rate
	amount = property(get_amount, set_amount)


class ValueP(PAC.Presentation):
	"""generic value presentation."""
	
	def __init__(self, controller=None, unit="€", frame=None):
		PAC.Presentation.__init__(self, controller)
		if frame == None:
			toplevel = Toplevel()
			frame = Frame(toplevel)
			frame.pack(expand=True, fill=X)
		
		button = Button(frame, text=unit, command=self.controller.edit_rate)
		button.pack(side=RIGHT, fill=BOTH)
		# euros rate is not editable
		if unit == "€":
			button.configure(state=DISABLED)
		
		self.entry = Entry(frame, justify=RIGHT)
		self.entry.pack(expand=True, fill=BOTH)
		self.entry.bind("<Return>", self.validate)
		
	def validate(self, _):
		self.controller.amount = self.value
		
	def get_value(self):
		return float(self.entry.get())
	def set_value(self, v):
		self.entry.delete(0, END)
		self.entry.insert(0, v)
	value = property(get_value, set_value)


class RateA(PAC.Abstraction):
	"""Rate abstraction."""
	
	def __init__(self, controller=None, rate=1.):
		PAC.Abstraction.__init__(self, controller)
		self._rate = rate
		
	def get_rate(self):
		return self._rate
	def set_rate(self, rate):
		self._rate = rate
		self.controller.update()
	rate = property(get_rate, set_rate)
		

# Rate edition ###############################################################

class RateEditorC(PAC.Controller):
	"""rate for a change."""
	def __init__(self, parent, unit):
		PAC.Controller.__init__(self, 
		                        parent=parent)
		self.presentation = RateEditorP(self, unit)

	def get_rate(self):
		return self.parent.rate
	def set_rate(self, rate):
		self.parent.rate = rate
	rate = property(get_rate, set_rate)
		
	
class RateEditorP(PAC.Presentation):
	def __init__(self, controller, unit):
		PAC.Presentation.__init__(self, controller=controller)
		toplevel = Toplevel()
		toplevel.title("%s rate" % unit)
		
		frame = Frame(toplevel)
		frame.pack(expand=True, fill=X)

		label = Label(frame, text="%s/€" % unit)
		label.pack(side=RIGHT, fill=BOTH)

		self.entry = Entry(frame, justify=RIGHT, text=self.controller)
		self.entry.pack(expand=True, fill=BOTH)
		self.entry.bind("<Return>", self.validate)
		
		self.value = self.controller.rate
		
	def validate(self, _):
		self.controller.rate = self.value

	def get_value(self):
		return float(self.entry.get())
	def set_value(self, v):
		self.entry.delete(0, END)
		self.entry.insert(0, v)
	value = property(get_value, set_value)
		
		
# test & main ################################################################
	
def _test():
	root.withdraw()
	euro = MoneyMVC()
	euro.children.append(MoneyC(euro, unit="$", rate=2.))
	
	return root.mainloop()
	
	
if __name__ == "__main__":
	sys.exit(_test())
	
		
