#!/usr/bin/env python
# -*- coding: utf-8 -*-

# imports ####################################################################

import sys

from Tkinter import *
root = Tk()
READONLY = "readonly"

sys.path.append("..")
import PAC

from calculator import *


# calculator #################################################################

class CalculatorA(PAC.Abstraction, Calculator):
	"""Caculator abstraction."""
	
	def __init__(self, controller=None):
		PAC.Abstraction.__init__(self, controller)
		Calculator.__init__(self)
		
		
class CalculatorP(PAC.Presentation):
	"""Caculator presentation."""
	
	def __init__(self, controller=None, window=root):
		PAC.Presentation.__init__(self, controller)
		
		self.window = window
		self.window.title("paccalc")

		self.entry = Entry(self.window, justify=RIGHT, state=READONLY)
		self.entry.pack(fill=X, padx=5, pady=5)
		
		self.build_keyboard()


	def get_value(self):
		return self.entry.get()
	def set_value(self, v):
		self.entry.configure(state=NORMAL)
		self.entry.delete(0, END)
		self.entry.insert(0, v)
		self.entry.configure(state=READONLY)
		self.new_value = True
	value = property(get_value, set_value)
		
		
	def build_keyboard(self):		
		"""building keyboard."""
		
		frame = Frame(self.window)
		frame.pack(expand=True, fill=BOTH, padx=5, pady=5)

		buttons = [
			[RST,  None, None, DIV],
			["7",  "8",  "9",  MUL],
			["4",  "5",  "6",  SUB],
			["1",  "2",  "3",  ADD],
			["0",  ".",  NEG,  EQU]
		]
		
		bindings = { RST:"Cc",
		             ".":".," }

		labels = { MUL:"×",
		           DIV:"÷",
		           NEG:"±",
		           ".":"•",
		           RST:"C" }
		           
		def button_cb(name):
			"""build appropriate callback for buttons."""
			if name in [ADD, SUB, MUL, DIV, EQU, NEG, RST]:
				def cb(): self.handle_command(name)
			else:
				def cb(): self.handle_digit(name)
			return cb
		
		def button_bindings(button):
			def press(_):
				button.configure(state=ACTIVE)
			def release(_):
				button.invoke()
				button.configure(state=NORMAL)
			return (press, release)
			
		for j, line in enumerate(buttons):
			for i, name in enumerate(line):
				if name == None: 
					continue
				button = Button(frame, text=labels.get(name, name), 
				                       command=button_cb(name))
				button.grid(row=j, column=i)
				
				# todo ajout du validate
				for c in bindings.get(name, name):
					press, release = button_bindings(button)
					self.window.bind("<KeyPress-%s>" % c, press)
					self.window.bind("<KeyRelease-%s>" % c, release)
					
		press, release = button_bindings(button)
		self.window.bind("<KeyPress-Return>", press)	
		self.window.bind("<KeyRelease-Return>", release)	
				
				
	def handle_command(self, n):
	 	self.controller.interpret(self.value)
	 	self.controller.interpret(n)
		
	def handle_digit(self, n):
		if n == "." and "." in self.value: 
			return
		self.entry.configure(state=NORMAL)
		if self.new_value:
			self.entry.delete(0, END)
		self.entry.insert(END, n)
		self.entry.configure(state=READONLY)
		self.new_value = False
				

class CalculatorC(PAC.Controller):
	"""Calculator controller.
	
	The controller simply forwards presentation inputs to the abstraction
	and resulting abstraction state change to the presentation.
	"""
	
	def __init__(self, parent=None):
		PAC.Controller.__init__(self,
		                        parent=parent,
		                        abstraction=CalculatorA(self),
		                        presentation=CalculatorP(self))
		self.interpret(RST)

	def interpret(self, n):
		"""read, eval, print."""
		self.presentation.value = self.abstraction.interpret(n)
		

# main #######################################################################

def main():
	paccalc = CalculatorC()
	return root.mainloop()
	
	
if __name__ == "__main__":
	sys.exit(main())
	